%{

/*
 * demo.y -- demonstration program
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

int yylex(void);
void yyerror(char *msg);

double memory[26];

%}

%union {
  double value;
  int index;
}

%token	<value>	NUMBER
%token	<index>	VAR
%token	<none>	NEWLINE
%token	<none>	PLUS MINUS STAR SLASH
%token	<none>	LPAREN RPAREN
%token	<none>	STORE

%type	<none>	list
%type	<value>	assign
%type	<value>	expr
%type	<value>	term
%type	<value>	factor
%type	<value>	primary

%start	list


%%


list		: /* empty */
		  {
		    /* nothing to do here */
		  }
		| list NEWLINE
		  {
		    /* nothing to do here */
		  }
		| list assign NEWLINE
		  {
		    printf("\t%.8g\n", $2);
		  }
		;

assign		: expr
		  {
		    $$ = $1;
		  }
		| assign STORE VAR
		  {
		    memory[$3] = $1;
		    $$ = $1;
		  }
		;

expr		: term
		  {
		    $$ = $1;
		  }
		| expr PLUS term
		  {
		    $$ = $1 + $3;
		  }
		| expr MINUS term
		  {
		    $$ = $1 - $3;
		  }
		;

term		: factor
		  {
		    $$ = $1;
		  }
		| term STAR factor
		  {
		    $$ = $1 * $3;
		  }
		| term SLASH factor
		  {
		    if ($3 == 0.0) {
		      printf("division by zero\n");
		      exit(99);
		    }
		    $$ = $1 / $3;
		  }
		;

factor		: primary
		  {
		    $$ = $1;
		  }
		| PLUS factor
		  {
		    $$ = $2;
		  }
		| MINUS factor
		  {
		    $$ = -$2;
		  }
		;

primary		: NUMBER
		  {
		    $$ = $1;
		  }
		| VAR
		  {
		    $$ = memory[$1];
		  }
		| LPAREN expr RPAREN
		  {
		    $$ = $2;
		  }
		;


%%


void yyerror(char *msg) {
  printf("%s\n", msg);
}


int main(int argc, char *argv[]) {
  int i;

  printf("Demo started:\n");
  for (i = 0; i < 26; i++) {
    memory[i] = 0.0;
  }
  yyparse();
  printf("Demo stopped.\n");
  return 0;
}
